﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Controls.Charts;
using Infragistics.Windows;
using System.ComponentModel;

namespace CahrtInListView
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            DataContext = ChartData.GenerateData(10);
        }
    }

    #region Tooltip Behavior

    public partial class CustomToolTipBehavior : DependencyObject
    {
        public static readonly DependencyProperty CustomToolTipProperty =
            DependencyProperty.RegisterAttached("CustomToolTip",
            typeof(CustomToolTip), typeof(CustomToolTipBehavior),
            new PropertyMetadata(null,
                (o, e) =>
                {
                    CustomToolTipChanged(o as XamDataChart,
                      e.OldValue as CustomToolTip,
                      e.NewValue as CustomToolTip);
                }));

        public static CustomToolTip GetCustomToolTip(DependencyObject target)
        {
            return target.GetValue(CustomToolTipProperty) as CustomToolTip;
        }

        public static void SetCustomToolTip(DependencyObject target, CustomToolTip value)
        {
            target.SetValue(CustomToolTipProperty, value);
        }
           

        public static void CustomToolTipChanged(
            XamDataChart chart,
            CustomToolTip oldVal,
            CustomToolTip newVal)
        {
            if (chart == null)
            {
                return;
            }
            if (oldVal != null)
            {
                oldVal.DetachOwner(chart);
            }
            if (newVal != null)
            {
                newVal.AttachOwner(chart);
            }
        }

    }

    public class CustomToolTip : DependencyObject, INotifyPropertyChanged
    {
        #region Private Fields
        private ToolTipDataContext context;

        private Border _toolTip;
        private ContentControl _content;
        private Canvas _container;
        private XamDataChart _owner;
        #endregion

        #region Initializing Methods
        public void AttachOwner(XamDataChart chart)
        {
            if (_owner != null)
            {
                DetachOwner(_owner);
            }

            _owner = chart;
            _owner.MouseMove += _owner_MouseMove;
            _owner.LayoutUpdated += _owner_LayoutUpdated;

            DataContext = new ToolTipDataContext();

            _toolTip = new Border();
            _toolTip.SetBinding(Border.StyleProperty,
                new Binding
                {
                    Source = this,
                    Path = new PropertyPath("BorderStyle")
                });


            _content = new ContentControl();
            _content.Content = DataContext;
            _content.SetBinding(ContentControl.ContentTemplateProperty,
                new Binding
                {
                    Source = this,
                    Path = new PropertyPath("ToolTipTemplate"),
                });
            _toolTip.Child = _content;
            
            _container = new Canvas();
            _container.Children.Add(_toolTip);
            Panel.SetZIndex(_container, 100);
        }

        public void DetachOwner(XamDataChart chart)
        {
            if (_owner != chart)
            {
                return;
            }

            chart.MouseMove -= _owner_MouseMove;
            chart.LayoutUpdated -= _owner_LayoutUpdated;

            //Grid parent = _container.Parent as Grid;
            //parent.Children.Remove(_container);

            _owner = null;
        }
        #endregion

        #region Properties

        public DataTemplate ToolTipTemplate
        {
            get { return (DataTemplate)GetValue(ToolTipTemplateProperty); }
            set { SetValue(ToolTipTemplateProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ToolTipTemplate.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ToolTipTemplateProperty =
            DependencyProperty.Register("ToolTipTemplate", 
            typeof(DataTemplate), 
            typeof(CustomToolTip), 
            new UIPropertyMetadata(
                null,
                new PropertyChangedCallback(
                    (o,e)=>
                    {
                        (o as CustomToolTip).NotifyPropertyChanged("ToolTipTemplate");
                    })));



        public Style BorderStyle
        {
            get { return (Style)GetValue(BorderStyleProperty); }
            set { SetValue(BorderStyleProperty, value); }
        }

        // Using a DependencyProperty as the backing store for BorderStyle.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty BorderStyleProperty =
            DependencyProperty.Register("BorderStyle",
            typeof(Style),
            typeof(CustomToolTip),
            new UIPropertyMetadata(
                null,
                new PropertyChangedCallback(
                    (o, e) =>
                    {
                        (o as CustomToolTip).NotifyPropertyChanged("BorderStyle");
                    })));




        public ToolTipDataContext DataContext
        {
            get { return context; }
            set { context = value; NotifyPropertyChanged("DataContext"); }
        }


        #endregion

        #region Helper Methods

        public void Refresh()
        {
            if (_owner != null)
            {
                Line crosshair =
                    Utilities.GetDescendantFromType(_owner, typeof(Line), false) as Line;


                if (crosshair != null && crosshair.Visibility == Visibility.Visible)
                {
                    Point crossPoint = new Point(Mouse.GetPosition(_owner).X - 60, _owner.ActualHeight);

                    _toolTip.Visibility = Visibility.Visible;

                    Canvas.SetLeft(_toolTip, crossPoint.X - _toolTip.ActualWidth / 2);

                    Canvas.SetTop(_toolTip, 0);

                    //Setting DataContext 
                    var xAxis = _owner.Axes.Where(a =>
                        a is NumericXAxis
                            || a is CategoryXAxis
                            || a is CategoryDateTimeXAxis).FirstOrDefault();
                    var yAxis = _owner.Axes.Where(a =>
                       a is NumericYAxis
                           || a is CategoryYAxis).FirstOrDefault();
                    if (xAxis != null && yAxis != null)
                    {
                        var viewport = new Rect(0, 0, xAxis.ActualWidth, yAxis.ActualHeight);
                        var window = _owner.WindowRect;
                        double x = xAxis.GetUnscaledValue(crossPoint.X, window, viewport);
                        double y = yAxis.GetUnscaledValue(crossPoint.Y, window, viewport);

                        DataContext.ValueX = x;
                        DataContext.ValueY = y;

                        // _content.Content = DataContext;
                    }

                }
            }
            else
            {
                _toolTip.Visibility = Visibility.Collapsed;
            }
        }
        

        #endregion

        #region Event Handlers
        void _owner_LayoutUpdated(object sender, EventArgs e)
        {
            if (_container.Parent == null)
            {
                ContentPresenter presenter = 
                    Utilities.GetDescendantFromName(_owner, "ContentPresenter") as ContentPresenter;

                if (presenter != null)
                {
                    Border chartContent =
                        Utilities.GetDescendantFromType(
                        presenter, typeof(Border), false) as Border;

                    if (chartContent != null)
                    {
                        (chartContent.Child as Grid).Children.Add(_container);
                    }
                }
            }
            Refresh();
        }

        void _owner_MouseMove(object sender, MouseEventArgs e)
        {
            Refresh();
        }
        #endregion

        #region INotifyPropertyChanged Memebers
        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
        #endregion
    }

    public class ToolTipDataContext : INotifyPropertyChanged
    {
        private double x;
        private double y;

        public double ValueX
        {
            get { return x; }
            set { x = value; NotifyPropertyChanged("ValueX"); }
        }

        public double ValueY
        {
            get { return y; }
            set { y = value; NotifyPropertyChanged("ValueY"); }
        }


        #region INotifyPropertyChanged Memebers
        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
        #endregion
    }

    #endregion    
    public class ChartData
    {
        public DateTime Date { get; set; }
        public double Value1 { get; set; }
        public double Value2 { get; set; }
        public double Value3 { get; set; }
        

        public static ObservableCollection<ChartData> GenerateData(int count)
        {
            ObservableCollection<ChartData> data = new ObservableCollection<ChartData>();

            Random r = new Random();

            ChartData data0 = new ChartData
            {
                Date = DateTime.Now,
                Value1 = r.Next(20, 40),
                Value2 = r.Next(20, 40),
                Value3 = r.Next(20, 40),
            };

            data.Add(data0);

            for (int i = 1; i < count; i++)
            {
                ChartData dat = new ChartData
                {
                    Date = DateTime.Now.AddDays(i),
                };

                if (r.NextDouble() > .5)
                {
                    dat.Value1 = data[i - 1].Value1 + r.NextDouble();
                    dat.Value2 = data[i - 1].Value2 + r.NextDouble();
                    dat.Value3 = data[i - 1].Value3 + r.NextDouble();
                }
                else
                {
                    dat.Value1 = data[i - 1].Value1 - r.NextDouble();
                    dat.Value2 = data[i - 1].Value2 - r.NextDouble();
                    dat.Value3 = data[i - 1].Value3 - r.NextDouble();
                }

                data.Add(dat);
            }

            return data;
        }
    }
    
    public class TicksToDateTimeConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (value.GetType() == typeof(double))
            {
                long datetime = (long)System.Convert.ToUInt64(value);
                DateTime dt = new DateTime(datetime);
                return dt;
            }

            return Binding.DoNothing;
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
